import os
import unittest
from mock import patch
from elifecleaner import pdf_utils, zip_lib
from tests.helpers import delete_files_in_folder

PDFIMAGES_OUTPUT = (
    b"page   num  type   width height color comp bpc  enc interp  object ID x-ppi y-ppi size ratio\n"
    b"--------------------------------------------------------------------------------------------\n"
    b"   1     0 image    1254  2131  rgb     3   8  image  no         9  0   220   220  139K 1.8%\n"
    b"   2     1 image    1314   358  rgb     3   8  image  no        12  0   223   222 18.0K 1.3%\n"
    b"   2     2 image    1324   361  rgb     3   8  image  no        13  0   224   224 21.0K 1.5%\n"
    b"   2     3 image    1319   360  rgb     3   8  image  no        14  0   223   224 21.5K 1.5%\n"
    b"   2     4 image    1319   360  rgb     3   8  image  no        15  0   223   224 19.0K 1.4%\n"
    b"   2     5 image    1314   358  rgb     3   8  image  no        16  0   223   222 18.5K 1.3%\n"
    b"   3     6 image    1321   360  rgb     3   8  image  no        19  0   224   223 23.5K 1.7%\n"
    b"   3     7 image    1328   362  rgb     3   8  image  no        20  0   225   225 23.5K 1.7%\n"
    b"   3     8 image    1328   362  rgb     3   8  image  no        21  0   225   225 20.8K 1.5%\n"
    b"   3     9 image    1324   361  rgb     3   8  image  no        22  0   224   224 21.9K 1.6%\n"
    b"   3    10 image    1328   362  rgb     3   8  image  no        23  0   225   225 23.9K 1.7%\n"
    b"   4    11 image    1330   363  rgb     3   8  image  no        26  0   225   225 21.5K 1.5%\n"
    b"   4    12 image    1326   362  rgb     3   8  image  no        27  0   225   225 24.9K 1.8%\n"
    b"   4    13 image    1326   362  rgb     3   8  image  no        28  0   225   225 26.3K 1.9%\n"
    b"   4    14 image    1331   360  rgb     3   8  image  no        29  0   225   225 29.7K 2.1%\n"
    b"   4    15 image    1330   363  rgb     3   8  image  no        30  0   225   225 24.2K 1.7%\n"
    b"   5    16 image    1321   360  rgb     3   8  image  no        33  0   224   223 25.0K 1.8%\n"
    b"   5    17 image    1312   358  rgb     3   8  image  no        34  0   222   222 23.6K 1.7%\n"
    b"   5    18 image    1328   362  rgb     3   8  image  no        35  0   225   225 25.0K 1.8%\n"
    b"   5    19 image    1324   361  rgb     3   8  image  no        36  0   224   224 23.1K 1.6%\n"
    b"   5    20 image    1307   356  rgb     3   8  image  no        37  0   221   221 26.1K 1.9%\n"
    b"   6    21 image    1322   358  rgb     3   8  image  no        40  0   220   220 26.6K 1.9%\n"
    b"   6    22 image    1300   352  rgb     3   8  image  no        41  0   220   220 23.6K 1.8%\n"
    b"   6    23 image    1300   352  rgb     3   8  image  no        42  0   220   220 35.3K 2.6%\n"
    b"   6    24 image    1300   352  rgb     3   8  image  no        43  0   220   220 20.0K 1.5%\n"
    b"   6    25 image    1300   352  rgb     3   8  image  no        44  0   220   220 29.6K 2.2%\n"
)


class Result:
    "struct for testing"

    def __init__(self):
        self.stdout = None


class TestPdfImagePages(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_pdfimages_output(self):
        zip_file = "tests/test_data/30-01-2019-RA-eLife-45644.zip"
        zip_lib.unzip_zip(zip_file, self.temp_dir)
        # run only if pdfimages is available
        if pdf_utils.pdfimages_exists():
            pdf_file = os.path.join(
                self.temp_dir, "30-01-2019-RA-eLife-45644/Appendix 1figure 10.pdf"
            )
            result = pdf_utils.pdfimages_output(pdf_file)
            self.assertIsNotNone(result.stdout)

    @patch.object(pdf_utils, "pdfimages_output")
    def test_pdf_image_pages(self, mock_pdfimages_output):
        mock_result = Result()
        mock_result.stdout = PDFIMAGES_OUTPUT
        mock_pdfimages_output.return_value = mock_result
        expected = {1, 2, 3, 4, 5, 6}
        pdf_file = "figure.pdf"
        pages = pdf_utils.pdf_image_pages(pdf_file)
        self.assertEqual(pages, expected)



import os
import unittest
from elifecleaner import configure_logging, LOGGER
from tests.helpers import delete_files_in_folder


class TestInit(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_configure_logging(self):
        expected = "INFO elifecleaner:test_init:test_configure_logging: test_configure_logging\n"
        log_handler = configure_logging(self.log_file)
        LOGGER.info("test_configure_logging")
        with open(self.log_file, "r") as open_file:
            self.assertEqual(open_file.read(), expected)
        LOGGER.removeHandler(log_handler)



import re


def pad_msid(msid):
    "zerofill string for article_id value"
    return "{:05d}".format(int(msid))


# match ascii characters from decimal 0 to 31, as hexidecimal character entitiy strings
# e.g. &#x001D; or &#x01;
CONTROL_CHARACTER_ENTITY_MATCH_PATTERN = r"&#x0{0,2}[0-1][0-9A-Fa-f];"
# string to replace character entities with
CONTROL_CHARACTER_ENTITY_REPLACEMENT = "_____"


def match_control_character_entities(string):
    "search the string for character entities of XML-incompatible control characters"
    match_pattern = re.compile(CONTROL_CHARACTER_ENTITY_MATCH_PATTERN)
    return match_pattern.findall(string)


def replace_control_character_entities(string):
    "replace character entities of control characters in the string"
    match_pattern = re.compile(CONTROL_CHARACTER_ENTITY_MATCH_PATTERN)
    return match_pattern.sub(CONTROL_CHARACTER_ENTITY_REPLACEMENT, string)


def match_control_characters(string):
    "search the string for XML-incompatible control characters"
    # char 9 is newline, 10 is tab, 13 is carriage return
    allowed = [9, 10, 13]
    return [char for char in string[:] if ord(char) <= 31 and ord(char) not in allowed]


def replace_control_characters(string):
    "replace control characters in the string"
    for char in list(set(match_control_characters(string))):
        string = string.replace(char, CONTROL_CHARACTER_ENTITY_REPLACEMENT)
    return string



import unittest
from elifecleaner import utils


class TestPadMsid(unittest.TestCase):
    def test_pad_msid(self):
        self.assertEqual(utils.pad_msid(666), "00666")
        self.assertEqual(utils.pad_msid("666"), "00666")


class TestMatchControlCharacterEntities(unittest.TestCase):
    def test_match_control_character_entities(self):
        self.assertEqual([], utils.match_control_character_entities(""))
        self.assertEqual(
            ["&#x001C;"], utils.match_control_character_entities("&#x001C;")
        )
        self.assertEqual(
            ["&#x001C;"], utils.match_control_character_entities("aaaa&#x001C;--")
        )
        self.assertEqual(
            ["&#x001E;", "&#x001E;"],
            utils.match_control_character_entities(" &#x001E;--&#x001E;"),
        )


class TestReplaceControlCharacterEntities(unittest.TestCase):
    def test_replace_control_character_entities(self):
        # empty string
        self.assertEqual("", utils.replace_control_character_entities(""))
        # one entity
        self.assertEqual(
            utils.CONTROL_CHARACTER_ENTITY_REPLACEMENT,
            utils.replace_control_character_entities("&#x001C;"),
        )
        # multiple entities
        self.assertEqual(
            utils.CONTROL_CHARACTER_ENTITY_REPLACEMENT * 4,
            utils.replace_control_character_entities("&#x00;&#x001C;&#x001D;&#x001E;"),
        )
        # entity found inside a string
        string_base = "<title>To %snd odd entities.</title>"
        string = string_base % "&#x001D;"
        expected = string_base % utils.CONTROL_CHARACTER_ENTITY_REPLACEMENT
        self.assertEqual(expected, utils.replace_control_character_entities(string))



import re
from xml.etree import ElementTree
from xml.etree.ElementTree import SubElement
import yaml
from elifecleaner import LOGGER


ASSESSMENT_TERMS_YAML = "assessment_terms.yaml"


def load_yaml(file_path=None):
    # load config from the file_path YAML file
    if not file_path:
        file_path = ASSESSMENT_TERMS_YAML
    try:
        with open(file_path, "r") as open_file:
            return yaml.load(open_file.read(), Loader=yaml.FullLoader)
    except:
        LOGGER.exception(
            "Exception in load_yaml for file_path: %s",
            file_path,
        )
        return None


def terms_data_from_yaml(file_path=None):
    return load_yaml(file_path)


def terms_data_by_terms(terms):
    "return a list of terms data which match the terms"
    terms_data = terms_data_from_yaml()
    if not terms_data:
        return {}
    # case-insensitive term matching using set operations to return a dict of matched YAML terms
    # if any of the supplied terms are found in the list of terms in the YAML then it is a hit
    return {
        term: data
        for term, data in terms_data.items()
        if set([term.lower() for term in terms]).intersection(
            set([term.lower() for term in data.get("terms", [])])
        )
    }


def terms_from_yaml():
    "get a list of unique terms from the YAML file"
    terms_data = terms_data_from_yaml()
    if not terms_data:
        return []
    terms_list = []
    for data in terms_data.values():
        terms_list += data.get("terms", [])
    return sorted(list(set(terms_list)))


def add_assessment_terms(root):
    "wrap specific terms with a bold tag in the body and add kwd tags"
    body_tag = root.find(".//body")
    if body_tag is not None:
        # list of terms from the YAML file
        terms = terms_from_yaml()

        # find matched terms for adding kwd tags later on
        matched_terms = []
        xml_string = ElementTree.tostring(body_tag, encoding="utf-8")
        for term in terms:
            term_match_groups = term_matches(xml_string.decode("utf-8"), term)
            for matched_term in term_match_groups:
                matched_terms.append(matched_term)

        # wrap terms with bold tags
        for tag_index, child_tag in enumerate(body_tag.iterfind("*")):
            # convert the tag to a string
            xml_string = ElementTree.tostring(child_tag, encoding="utf-8")
            # wrap terms with bold tag
            xml_string = xml_string_term_bold(xml_string.decode("utf-8"), terms)
            # convert the XML string back to an element
            new_child_tag = ElementTree.fromstring(xml_string)
            # remove old tag
            body_tag.remove(child_tag)
            # insert the new tag
            body_tag.insert(tag_index, new_child_tag)

        # add kwd-group and kwd tags
        terms_data = terms_data_by_terms(matched_terms)

        if terms_data:
            front_stub_tag = root.find(".//front-stub")

            # tags will be added in the order listed in the YAML file
            group_to_kwd_group_tag_map = {}
            for key, data in terms_data.items():
                if data.get("group") in group_to_kwd_group_tag_map.keys():
                    # reuse the kwd-group tag
                    kwd_group_tag = group_to_kwd_group_tag_map.get(data.get("group"))
                else:
                    # create a new kwd-group tag
                    kwd_group_tag = SubElement(front_stub_tag, "kwd-group")
                    kwd_group_tag.set("kwd-group-type", data.get("group"))
                    group_to_kwd_group_tag_map[data.get("group")] = kwd_group_tag
                # add a kwd tag
                kwd_tag = SubElement(kwd_group_tag, "kwd")
                kwd_tag.text = key


def term_match_pattern(term_text):
    "regular expression to find mentions of a term"
    return r"[\s\W](%s)\W" % (term_text)


def term_matches(xml_string, term_text):
    "get list of terms in text that are not already preceeded by the bold tag"
    return re.findall(term_match_pattern(term_text), xml_string, flags=re.IGNORECASE)


def xml_string_term_bold(xml_string, terms):
    "wrap occurences of each term in the XML string with a <bold> tag"
    open_tag = "<bold>"
    close_tag = "</bold>"
    for term in terms:
        if not term:
            continue
        # look for term in the text but not preceeded by a bold open tag
        term_match_groups = term_matches(xml_string, term)
        for matched_term in term_match_groups:
            safe_match_pattern = r"(?<!%s)(%s)(\W)" % (open_tag, matched_term)
            replacement_pattern = r"%s\1%s\2" % (
                open_tag,
                close_tag,
            )
            xml_string = re.sub(
                safe_match_pattern,
                replacement_pattern,
                xml_string,
                count=1,
                flags=re.IGNORECASE,
            )

    return xml_string



import unittest
import os
import sys
import time
import zipfile
from collections import OrderedDict
from xml.etree import ElementTree
from mock import patch
from elifearticle.article import Article, ArticleDate
from elifecleaner import video_xml
from tests.helpers import delete_files_in_folder, read_fixture

JOURNAL_DATA = {
    "journal_ids": {"nlm-ta": "elife", "publisher-id": "eLife"},
    "journal_title": "eLife",
    "publisher_name": "eLife Sciences Publications, Ltd",
}

VIDEO_DATA = [
    OrderedDict(
        [
            ("upload_file_nm", "Video 1 AVI.avi"),
            ("video_id", "video1"),
            ("video_filename", "elife-64719-video1.avi"),
        ]
    )
]


def build_article(doi=None, manuscript=None):
    article = Article(doi=doi)
    article.manuscript = manuscript
    return article


class TestSetPubDate(unittest.TestCase):
    def test_set_pub_date(self):
        root = ElementTree.fromstring("<article/>")
        article = build_article()
        pub_type = "electronic"
        date_time = time.strptime("2022-04-19", "%Y-%m-%d")
        article.add_date(ArticleDate(pub_type, date_time))
        expected = (
            b"<article>"
            b'<pub-date date-type="%s">'
            b"<day>19</day>"
            b"<month>04</month>"
            b"<year>2022</year>"
            b"</pub-date>"
            b"</article>" % bytes(pub_type, "utf-8")
        )
        video_xml.set_pub_date(root, article, pub_type)
        xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, expected)


class TestSetArticleMeta(unittest.TestCase):
    def test_set_article_meta(self):
        root = ElementTree.fromstring("<article/>")
        article = build_article("10.7554/eLife.64719", 64719)
        expected = (
            b"<article>"
            b"<article-meta>"
            b'<article-id pub-id-type="publisher-id">64719</article-id>'
            b'<article-id pub-id-type="doi">10.7554/eLife.64719</article-id>'
            b"</article-meta>"
            b"</article>"
        )
        video_xml.set_article_meta(root, article)
        xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, expected)


class TestSetFront(unittest.TestCase):
    def test_set_front(self):
        root = ElementTree.fromstring("<article/>")
        article = build_article("10.7554/eLife.64719", 64719)
        expected = (
            b"<article>"
            b"<front>"
            b"<journal-meta>"
            b'<journal-id journal-id-type="nlm-ta">elife</journal-id>'
            b'<journal-id journal-id-type="publisher-id">eLife</journal-id>'
            b"<journal-title-group>"
            b"<journal-title>eLife</journal-title>"
            b"</journal-title-group>"
            b"<publisher>"
            b"<publisher-name>eLife Sciences Publications, Ltd</publisher-name>"
            b"</publisher>"
            b"</journal-meta>"
            b"<article-meta>"
            b'<article-id pub-id-type="publisher-id">64719</article-id>'
            b'<article-id pub-id-type="doi">10.7554/eLife.64719</article-id>'
            b"</article-meta>"
            b"</front>"
            b"</article>"
        )
        video_xml.set_front(root, JOURNAL_DATA, article)
        xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, expected)


class TestBody(unittest.TestCase):
    def test_set_body(self):
        root = ElementTree.fromstring("<article/>")
        media_tag_xml_string = (
            b'<media xlink:href="elife-64719-video1.avi" id="video1" '
            b'content-type="glencoe play-in-place height-250 width-310" mimetype="video" />'
        )
        if sys.version_info < (3, 8):
            # pre Python 3.8 tag attributes are automatically alphabetised
            media_tag_xml_string = (
                b'<media content-type="glencoe play-in-place height-250 width-310" '
                b'id="video1" mimetype="video" xlink:href="elife-64719-video1.avi" />'
            )
        expected = (
            b"<article>" b"<body>" + media_tag_xml_string + b"</body>" b"</article>"
        )
        video_xml.set_body(root, VIDEO_DATA)
        xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, expected)


class TestOutputXml(unittest.TestCase):
    def test_output_xml(self):
        xml_string = "<article/>"
        root = ElementTree.fromstring(xml_string)
        expected = b'<?xml version="1.0" encoding="utf-8"?>' + bytes(
            xml_string, "utf-8"
        )
        output = video_xml.output_xml(root)
        self.assertEqual(output, expected)

    def test_output_xml_pretty(self):
        xml_string = "<article/>"
        root = ElementTree.fromstring(xml_string)
        expected = (
            b'<?xml version="1.0" encoding="utf-8"?>'
            + b"\n"
            + bytes(xml_string, "utf-8")
            + b"\n"
        )
        output = video_xml.output_xml(root, pretty=True)
        self.assertEqual(output, expected)


class TestGenerateXml(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.video_xml_64719 = read_fixture("video_xml_64719.xml", "rb")
        if sys.version_info < (3, 8):
            # pre Python 3.8 tag attributes are automatically alphabetised
            self.video_xml_64719 = self.video_xml_64719.replace(
                b'<article xmlns:mml="http://www.w3.org/1998/Math/MathML" xmlns:xlink="http://www.w3.org/1999/xlink" dtd-version="1.1d1" article-type="research-article">',
                b'<article article-type="research-article" dtd-version="1.1d1" xmlns:mml="http://www.w3.org/1998/Math/MathML" xmlns:xlink="http://www.w3.org/1999/xlink">',
            )
            self.video_xml_64719 = self.video_xml_64719.replace(
                b'<media xlink:href="elife-64719-video1.avi" id="video1" content-type="glencoe play-in-place height-250 width-310" mimetype="video"/>',
                b'<media content-type="glencoe play-in-place height-250 width-310" id="video1" mimetype="video" xlink:href="elife-64719-video1.avi"/>',
            )

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    @patch.object(time, "gmtime")
    def test_glencoe_xml(self, fake_gmtime):
        fake_gmtime.return_value = time.strptime("2022-04-19", "%Y-%m-%d")
        zip_file = "tests/test_data/08-11-2020-FA-eLife-64719.zip"
        xml_file_name = "08-11-2020-FA-eLife-64719/08-11-2020-FA-eLife-64719.xml"
        xml_file_path = os.path.join(self.temp_dir, xml_file_name)
        with zipfile.ZipFile(zip_file, "r") as input_zipfile:
            input_zipfile.extract(xml_file_name, self.temp_dir)
        xml_string = video_xml.glencoe_xml(xml_file_path, VIDEO_DATA)
        self.assertEqual(xml_string, self.video_xml_64719)

    @patch.object(time, "gmtime")
    def test_generate_xml(self, fake_gmtime):
        fake_gmtime.return_value = time.strptime("2022-04-19", "%Y-%m-%d")
        article = build_article("10.7554/eLife.64719", 64719)
        xml_string = video_xml.generate_xml(article, JOURNAL_DATA, VIDEO_DATA)
        self.assertEqual(xml_string, self.video_xml_64719)



from setuptools import setup

import elifecleaner

with open("README.md") as fp:
    readme = fp.read()

setup(
    name="elifecleaner",
    version=elifecleaner.__version__,
    description="Clean and transform article submission files into a consistent format.",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=["elifecleaner"],
    license="MIT",
    install_requires=[
        "docmaptools>=0.6.0",
        "elifetools",
        "elifearticle>=0.10.0",
        "jatsgenerator>=0.4.0",
        "PyYAML>=5.4.1",
        "wand >= 0.5.2",
    ],
    url="https://github.com/elifesciences/elife-cleaner",
    maintainer="eLife Sciences Publications Ltd.",
    maintainer_email="tech-team@elifesciences.org",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
    ],
)



import unittest
from xml.etree import ElementTree
from mock import patch
import yaml
from elifetools import xmlio
from elifecleaner import assessment_terms


class TestLoadYaml(unittest.TestCase):
    def test_load_yaml(self):
        "load YAML file using the default"
        result = assessment_terms.load_yaml()
        self.assertIsNotNone(result)
        self.assertTrue(isinstance(result, dict))

    def test_no_file(self):
        "test if no file is supplied also uses the default"
        result = assessment_terms.load_yaml(None)
        self.assertIsNotNone(result)
        self.assertTrue(isinstance(result, dict))

    @patch.object(yaml, "load")
    def test_exception(self, mock_load):
        "test an exception when loading the YAML file"
        mock_load.side_effect = Exception("An exception")
        result = assessment_terms.load_yaml()
        self.assertEqual(result, None)


class TestTermsDataByTerms(unittest.TestCase):
    def setUp(self):
        self.terms_data = {
            "Beagle": {"group": "dog", "terms": ["beagle", "beagles"]},
            "Dingo": {"group": "dog", "terms": ["dingo", "dingos"]},
            "Dodo": {"group": "bird", "terms": ["dodo", "dodos"]},
            "Parrot": {"group": "bird", "terms": ["parrot", "parrots"]},
        }

    @patch.object(assessment_terms, "terms_data_from_yaml")
    def test_terms_data_by_terms(self, mock_terms_data):
        mock_terms_data.return_value = self.terms_data
        terms = ["dodo", "dodos", "DINGO"]
        expected_keys = ["Dingo", "Dodo"]
        result = assessment_terms.terms_data_by_terms(terms)
        self.assertEqual(sorted(result.keys()), sorted(expected_keys))

    @patch.object(assessment_terms, "terms_data_from_yaml")
    def test_no_match(self, mock_terms_data):
        mock_terms_data.return_value = self.terms_data
        terms = ["penguin"]
        expected_keys = []
        result = assessment_terms.terms_data_by_terms(terms)
        self.assertEqual(sorted(result.keys()), sorted(expected_keys))

    @patch.object(assessment_terms, "terms_data_from_yaml")
    def test_no_data(self, mock_terms_data):
        mock_terms_data.return_value = None
        terms = ["penguin"]
        expected_keys = []
        result = assessment_terms.terms_data_by_terms(terms)
        self.assertEqual(sorted(result.keys()), sorted(expected_keys))

    @patch.object(assessment_terms, "terms_data_from_yaml")
    def test_none(self, mock_terms_data):
        mock_terms_data.return_value = None
        terms = ["dodo"]
        expected_keys = []
        result = assessment_terms.terms_data_by_terms(terms)
        self.assertEqual(sorted(result.keys()), sorted(expected_keys))


class TestTermsFromYaml(unittest.TestCase):
    def test_terms_from_yaml(self):
        self.assertTrue(len(assessment_terms.terms_from_yaml()) > 0)

    @patch.object(assessment_terms, "terms_data_from_yaml")
    def test_yaml_none(self, mock_terms_data):
        mock_terms_data.return_value = None
        self.assertEqual(assessment_terms.terms_from_yaml(), [])


class TestAddAssessmentTerms(unittest.TestCase):
    def setUp(self):
        # register XML namespaces
        xmlio.register_xmlns()

    def test_add_assessment_terms(self):
        "test editor-report example with specific terms in the abstract"
        xml_root = ElementTree.fromstring(
            b'<article xmlns:xlink="http://www.w3.org/1999/xlink"><front-stub><title-group>'
            b"<article-title>eLife assessment</article-title>"
            b"</title-group></front-stub>"
            b"<body>"
            b"<p>Landmark convincing evaluation "
            b'(<ext-link xlink:href="https://example.org"/>), '
            b"convincingly compelling if "
            b"incompletely unconvincing (convincingly).</p>"
            b"</body>"
            b"</article>"
        )

        expected_xml = (
            b'<article xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<front-stub>"
            b"<title-group>"
            b"<article-title>eLife assessment</article-title>"
            b"</title-group>"
            b'<kwd-group kwd-group-type="evidence-strength">'
            b"<kwd>Compelling</kwd>"
            b"<kwd>Convincing</kwd>"
            b"<kwd>Incomplete</kwd>"
            b"</kwd-group>"
            b'<kwd-group kwd-group-type="claim-importance">'
            b"<kwd>Landmark</kwd>"
            b"</kwd-group>"
            b"</front-stub>"
            b"<body><p><bold>Landmark</bold> <bold>convincing</bold> evaluation "
            b'(<ext-link xlink:href="https://example.org" />), '
            b"<bold>convincingly</bold> <bold>compelling</bold> if "
            b"<bold>incompletely</bold> unconvincing (<bold>convincingly</bold>).</p>"
            b"</body>"
            b"</article>"
        )

        assessment_terms.add_assessment_terms(xml_root)
        rough_xml_string = ElementTree.tostring(xml_root, "utf-8")
        # assert
        self.assertEqual(rough_xml_string, expected_xml)

    def test_multiple_p_tags(self):
        "test editor-report with multiple p tags in the abstract"
        xml_root = ElementTree.fromstring(
            b"<sub-article><front-stub><title-group>"
            b"<article-title>eLife assessment</article-title>"
            b"</title-group></front-stub>"
            b"<body>"
            b"<p>Landmark.</p>"
            b"<p><bold>Important</bold>!</p>"
            b"</body>"
            b"</sub-article>"
        )

        expected_xml = (
            b"<sub-article>"
            b"<front-stub>"
            b"<title-group><article-title>eLife assessment</article-title></title-group>"
            b'<kwd-group kwd-group-type="claim-importance">'
            b"<kwd>Important</kwd>"
            b"<kwd>Landmark</kwd>"
            b"</kwd-group>"
            b"</front-stub>"
            b"<body>"
            b"<p><bold>Landmark</bold>.</p>"
            b"<p><bold>Important</bold>!</p>"
            b"</body>"
            b"</sub-article>"
        )

        assessment_terms.add_assessment_terms(xml_root)
        rough_xml_string = ElementTree.tostring(xml_root, "utf-8")
        # assert
        self.assertEqual(rough_xml_string, expected_xml)

    def test_non_p_tag(self):
        "test for a tag which is not a p tag"
        xml_string = (
            b"<article>"
            b"<front-stub>"
            b"<title-group>"
            b"<article-title>eLife assessment</article-title>"
            b"</title-group>"
            b"</front-stub>"
            b"<body>"
            b"<sec>Landmark</sec>"
            b"</body>"
            b"</article>"
        )
        expected_xml = (
            b"<article>"
            b"<front-stub>"
            b"<title-group>"
            b"<article-title>eLife assessment</article-title>"
            b"</title-group>"
            b'<kwd-group kwd-group-type="claim-importance">'
            b"<kwd>Landmark</kwd>"
            b"</kwd-group>"
            b"</front-stub>"
            b"<body><sec><bold>Landmark</bold></sec></body>"
            b"</article>"
        )
        xml_root = ElementTree.fromstring(xml_string)
        assessment_terms.add_assessment_terms(xml_root)
        rough_xml_string = ElementTree.tostring(xml_root, "utf-8")
        self.assertEqual(rough_xml_string, expected_xml)


class TestXmlStringTermBold(unittest.TestCase):
    def test_xml_string_term_bold(self):
        test_data = {
            "xml_string": ("<p>In Author Response image 1, yes response.</p>"),
            "terms": ["response"],
            "expected": (
                "<p>In "
                "Author <bold>Response</bold> image 1, yes <bold>response</bold>.</p>"
            ),
        }
        modified_xml_string = assessment_terms.xml_string_term_bold(
            test_data.get("xml_string"), test_data.get("terms")
        )
        self.assertEqual(modified_xml_string, test_data.get("expected"))

    def test_blank_terms(self):
        "test for blank or missing terms"
        test_data = {
            "xml_string": ("<p>In Author Response image 1, yes response.</p>"),
            "terms": ["", None],
            "expected": ("<p>In " "Author Response image 1, yes response.</p>"),
        }
        modified_xml_string = assessment_terms.xml_string_term_bold(
            test_data.get("xml_string"), test_data.get("terms")
        )
        self.assertEqual(modified_xml_string, test_data.get("expected"))
